//---------------------------------------------------------------------------
#ifndef coreH
#define coreH

//---------------------------------------------------------------------------
#define DLLExport extern "C" _stdcall _export

//---------------------------------------------------------------------------
// ** Interface structures **
// These give access to Genesis's drawing model and fall into the following
// categories...
// 1.  Overall Project information and Grid definition
// 2.  Terrain layer definition
// 3.  Terrain structures regions, lines and points definition
// 4.  Vegetation definition
// 5.  Urban definition
// 6.  Camera/Target/Lighting definition
// 7.  General program functionality - refresh etc.
//
// Much of this corresponds directly to the dialog structure of G2, but
// to summerize...
//
//      +---------+        +------+
//      |         |--------| Grid |
//      |         |        +------+   +------------+
//      | Project |-------------------| Atmosphere |
//      |         |                   +----------- +
//      |         |-------------------|   Camera   |
//      +---------+                   +------------+
//       |   |   |
//       |   |   +-------------------------------------------+
//       |   |                                              /|\
//       |   |                               +-------------------------------+
//       |   |                               |          Terrain Layers       |
//       |   |                               +-------------------------------+
//       |   |                                      |         |         |
//       |   |                                     /|\       /|\       /|\
//       |   |                                  +-------+  +-----+  +------+
//       |   |                                  |Regions|  |Lines|  |Models|
//       |   |                                  +-------+  +-----+  +------+
//    +--+   +---+                                |   |     |   |
//    |          |                                |   |     |   |
//    |          |                                |   |     |   |
//    |          |                                |   |     |   |
//    |         /|\                               |   |     |   |
//    |  +---------------+\*4                     |   |     |   |
//    |  |               |------------------------+   |     |   |
//    |  |     Plants    |/                           |     |   |
//    |  |               |\*4                         |     |   |
//    |  |               |----------------------------------+   |
//    |  +---------------+/                           |         |
//   /|\                                              |         |
//  +---------------+                                 |         |
//  |               |---------------------------------+         |
//  |     Urban     |                                           |
//  |               |-------------------------------------------+
//  +---------------+
//
//                                      *region/lines limited to 4 - may change
//---------------------------------------------------------------------------

// project and grid parameter information
struct G2_PARAMETERS
{
  // grid
  int    GridWidth;          // Grid width
  int    GridHeight;         // Grid height
  double GridSpacingX;       // Grid point spacing along x
  double GridSpacingY;       // ...and y
  double MaxHeight;          // Grid max height
  double MinHeight;          // ...and min
  double VerticalScale;      // Vertical scaling
  double GeoRef[4];          // Georeferencing info, top,left:bottom,right
  // Terrain layers
  int    TerrainLayersCount; // number terrain layers
  int    PlantCount;         // number 2D plant images
  int    PolygonCount;       // Number of polygons
  int    LineCount;          // Number of lines
  int    ModelCount;         // Number of models
  // Directory paths
  char   PlantDir[256];      // Plant model directory
  char   ModelDir[256];      // 3D model directory
  // Reserved
  char   Reserved[512];      // reserved
};

// Camera, Target etc
struct G2_VIEWPARAMETERS
{
  float  CameraPosition[3];  // camera position xyz - y is vertical
  float  TargetPosition[3];  // target position xyz - y is vertical
  double Perspective;        // perspective in degrees
  double LightAltitude;      // Altitude position of light in degrees.  90 is overhead
  double LightAzimuth;       // Azimuth position of light in degrees.    0 is North
  char   Reserved[64];       // reserved
};

// Surface definition
struct G2_SDEF
{
  float  PrimaryColor[4];    // primary and secondary color series are fractal...
  float  SecondaryColor[4];  // ...mixed to generate the surface
  double Roughness;          // surface roughness
  double Specularity;        // lighting characteristics
  int    Shininess;          //
  double Emission;           //
  char   Reserved[64];       // reserved
};

// plant used on a terrain
struct G2_PLANT
{
  int    ID;                 // ID of plant - retrieve details with
  double Size;               // size and...
  double Density;            // ...density of plants plotted
  char   Reserved[64];       // reserved
};

// terrain layer definition
struct G2_TERRAINLAYER
{
  int      ID;                     // Unique key for this layer
  bool     OverrideDefaultColor;   // Override default surface colors
  bool     OverrideDefaultSurface; // Override default surface properties (other than colors)
  bool     DrawPlants;             // Draw plants in this layer
  char     Name[256];              // Name of this layer
  bool     DrawLayer;              // polygon drawn in landscape module
  int      SurfaceCount;           // Surfaces - must be 4 for this SDK version
  G2_SDEF  Surface[4];             // Surface definition
  int      PlantCount;             // Plant count - must be 4 for this SDK version
  G2_PLANT Plants[4];              // Plant definition
  char     ModelSource[256];       // 3D model source file - relative path
  double   LineWidth;              // polyline width
  double   ModelSize;              // default model size
  double   ModelRotation;          // default model rotation
  float    LandscapeColor[4];      // color layer is drawn IN LANDSCAPE MODULE
  bool     PolygonLineOnly;        // Draw plants on polygon edges only
  bool     DrawModel;              // Draw 3D models in this layer
  bool     DrawUrban;              // Draw urban plan
  int      UrbanID;                // Urban plan asscoiated with this terrain
  int      TerrainLayerType;       // Terrain layer type
  char     Reserved[508];          // reserved
};

// Actor info
struct G2_PLANTPARAMETERS
{
  int      ID;                // An ID - internal use by G2 only
  char     Name[256];         // Display name for this plant
  char     FarFileName[256];  // BMP source file - far
  char     MidFileName[256];  // BMP source file - mid
  char     NearFileName[256]; // BMP source file - near
  char     Reserved[64];      // reserved
};

// Urban plans
struct G2_URBANPLAN
{
  int      ID;                  // An ID - internal use by G2 only
  char     Name[256];           // Display name for this plan
  double   SizeX;               // x size in metres of plan
  double   SizeY;               // y size in metres of plan
  double   Height;              // max height in metres of plan
  char     HeightPlan[256];     // BMP source file - far
  char     WallColorPlan[256];  // BMP source file - mid
  char     RoofColorPlan[256];  // BMP source file - near
  int      Zone;                // Urban zone type
  char     Reserved[512];       // reserved
};

// Polygon info
// Note that responsibility for allocating memory for coordinates is the plugin programmers responsibility
struct G2_POLYGONINFO
{
  int      ID;              // An ID - internal use by G2 only
  bool     Deleted;         // Deleted flag.
  int      TerrainLayerID;  // ID of terrain layer this relates to
  int      CoordinateCount; // coordinates in poly (not including closing point)
  float    *Coordinates;    // Pointer to array of floats holding x,y
};

// Line info
// Note that responsibility for allocating memory for coordinates is the plugin programmers responsibility
struct G2_LINEINFO
{
  int      ID;              // An ID - internal use by G2 only
  bool     Deleted;         // Deleted flag.
  int      TerrainLayerID;  // ID of terrain layer this relates to
  int      CoordinateCount; // coordinates in poly (not including closing point)
  float    *Coordinates;    // Pointer to array of floats holding x,y
};

// Model info
struct G2_MODELINFO
{
  int      ID;              // An ID - internal use by G2 only
  bool     Deleted;         // Deleted flag.
  int      TerrainLayerID;  // ID of terrain layer this relates to
  float    x;               // x,y position - georeferenced
  float    y;
  float    Size;            // Model size and
  float    Rotation;        // ...Rotation
  char     Reserved[64];    // reserved
};

// ---- Callback Interface structure returned on plugin initialization -------//
// GensisisII is largely written in Delphi and hence this structure contains  //
// some additions to the normal C/C++ definition.                             //
// The functions reference the current project object in Genesis and carry a  //
// pascal self reference value as parameter 0.  This can safely be ignored    //
// and passed as NULL for C/C++ use.  Additional BYTEs handle 'of object'     //
// flag used by equivalent Delphi record                                      //
//----------------------------------------------------------------------------//
struct G2_INTERFACE
{
  // Get/Set basic project/grid parameters
  void   _stdcall (*GLGGetParameters)(void *self, G2_PARAMETERS *G2Parameters); BYTE f0;
  void   _stdcall (*GLGSetParameters)(void *self, G2_PARAMETERS *G2Parameters); BYTE f1;
  // Get/Set viewing parameters - camera, target etc.
  void   _stdcall (*GLGGetViewingParameters)(void *self, G2_VIEWPARAMETERS *G2ViewParameters); BYTE f2;
  void   _stdcall (*GLGSetViewingParameters)(void *self, G2_VIEWPARAMETERS *G2ViewParameters); BYTE f3;
  // Create grid and Get/Set grid point properties
  bool   _stdcall (*GLGCreateNewGrid)(void *self, int x, int y, float sx, float sy); BYTE f4;
  double _stdcall (*GLGGetPointHeight)(void *self, int x, int y); BYTE f5;
  void   _stdcall (*GLGSetPointHeight)(void *self, int x, int y, double NewHeight); BYTE f6;
  bool   _stdcall (*GLGGetPointWater)(void *self, int x, int y); BYTE f7;
  void   _stdcall (*GLGSetPointWater)(void *self, int x, int y, bool isWater); BYTE f8;
  bool   _stdcall (*GLGGetHeightField)(void *self, int Size, float *HeightField);  BYTE f27;
  bool   _stdcall (*GLGSetHeightField)(void *self, int Size, float *HeightField);  BYTE f28;
  // Create/Get/Set terrain layers
  bool   _stdcall (*GLGGetTerrainLayerInfo)(void *self, int LayerIdx, G2_TERRAINLAYER *G2TerrainLayer); BYTE f9;
  bool   _stdcall (*GLGSetTerrainLayerInfo)(void *self, int LayerIdx, G2_TERRAINLAYER *G2TerrainLayer); BYTE f10;
  int    _stdcall (*GLGCreateNewTerrainLayer)(void *self, G2_TERRAINLAYER *G2TerrainLayer); BYTE f11;
  // Create/Get/Set plant info
  bool   _stdcall (*GlGGetPlantInfo)(void *self, int PlantIdx, G2_PLANTPARAMETERS *G2PlantParameters); BYTE f12;
  bool   _stdcall (*GlGSetPlantInfo)(void *self, int PlantIdx, G2_PLANTPARAMETERS *G2PlantParameters); BYTE f13;
  int    _stdcall (*GlGCreateNewPlant)(void *self, G2_PLANTPARAMETERS *G2PlantParameters); BYTE f14;
  // Create/Get/Set urban plans
  bool   _stdcall (*GlGGetUrbanPlan)(void *self, int UrbanIdx, G2_URBANPLAN *PG2UrbanPlan); BYTE f212;
  bool   _stdcall (*GlGSetUrbanPlan)(void *self, int UrbanIdx, G2_URBANPLAN *PG2UrbanPlan); BYTE f213;
  int    _stdcall (*GlGCreateNewUrbanPlan)(void *self, G2_URBANPLAN *PG2UrbanPlan); BYTE f214;
  // Deletion functions for polygons, lines and models (points)
  bool   _stdcall (*GLGDeleteAllPolygonsInLayer)(void *self, int LayerIdx); BYTE f15;
  bool   _stdcall (*GLGDeleteAllLinesInLayer)(void *self, int LayerIdx); BYTE f16;
  bool   _stdcall (*GLGDeleteAllModelsInLayer)(void *self, int LayerIdx); BYTE f17;
  // Create polygons and Get info
  bool   _stdcall (*GLGGetPolygonInfo)(void *self, int PolygonIdx, G2_POLYGONINFO *G2PolygonInfo); BYTE f18;
  bool   _stdcall (*GLGGetPolygonCoordinates)(void *self, int PolygonIdx, G2_POLYGONINFO *G2PolygonInfo); BYTE f19;
  bool   _stdcall (*GLGAddPolygon)(void *self, G2_POLYGONINFO *G2PolygonInfo); BYTE f20;
  // Create lines and Get info
  bool   _stdcall (*GLGGetLineInfo)(void *self, int LineIdx, G2_LINEINFO *G2LineInfo); BYTE f21;
  bool   _stdcall (*GLGGetLineCoordinates)(void *self, int LineIdx, G2_LINEINFO *G2LineInfo); BYTE f22;
  bool   _stdcall (*GLGAddLine)(void *self, G2_LINEINFO *G2LineInfo); BYTE f23;
  // Create lines and Get info
  bool   _stdcall (*GLGGetModelInfo)(void *self, int ModelIdx, G2_MODELINFO *G2ModelInfo); BYTE f24;
  bool   _stdcall (*GLGAddModel)(void *self, G2_MODELINFO *G2ModelInfo); BYTE f25;
  // Utility function to force update
  void   _stdcall (*GLGUpdateChildren)(void *self); BYTE f26;
};

// Global interfaces for demo
G2_INTERFACE       G2INTERFACE;
G2_VIEWPARAMETERS  G2VIEW;
G2_PARAMETERS      G2PARAMETERS;
G2_TERRAINLAYER    G2TERRAIN;
G2_PLANTPARAMETERS G2PLANT;
G2_MODELINFO       G2MODEL;
G2_POLYGONINFO     G2POLYGON;
G2_LINEINFO        G2LINE;

// Exported functions.  These are called by GenesisII
DLLExport void DescribePlugin(char* description,int* version);
DLLExport void InitPlugin(G2_INTERFACE* g2);
DLLExport  int RunPlugin();

#endif
